import React, { Component } from 'react';
import DeckGL, { PolygonLayer, MapController } from 'deck.gl';
import { StaticMap } from 'react-map-gl';

import Tooltip from './Tooltip.js';

const polygon_size = 0.01
const polygon_size_lat = polygon_size * 2
const polygon_size_lon = polygon_size

// Viewport settings
const viewport = {
  longitude: 25.0,
  latitude: 62.217217,
  zoom: 6,
  pitch: 60,
  bearing: 0
};

class PointMap extends Component {

  constructor(props) {
    super(props);
    const mouseover = null;
    const clicked = null;
    this.state = { viewport, mouseover, clicked };
  }

  update_tooltip(visible, x, y, content) {
    this.setState((prevState) => {
      let tooltipVisible = visible;
      y -= 20;
      x += 10;
      let tooltip = {x, y, content};
      return {
        ...prevState,
        tooltipVisible,
        tooltip,
      };
    });
  }

  handleOnHover(e) {
    this.setState(prevState => {
      let mouseover = null;
      if (e.picked) {
        mouseover = e.object.get_id();
      }
      return { ...prevState, mouseover };
    });
  }

  handleOnClickObject(e) {
    this.setState(prevState => {
      let clicked = null;
      if (e.picked) {
        clicked = e.object.get_id();
      }
      return { ...prevState, clicked };
    });
    return false;
  }

  handleOnClickMap(e) {
    if (e === null) {
      this.setState(prevState => {
        const clicked = null;
        return { ...prevState, clicked };
      });
    }
  }

  render() {
    const point_coords = this.props.points.toArray();
    const layers = [
      new PolygonLayer({
        id: 'lam-points',
        data: point_coords,
        getPosition: d => d.get_coordinates(),
        getFillColor: d => d.get_color(),
        getPolygon: d => {
          let [lat, lon] = d.coordinates;
          let coords = [
            [lat + -polygon_size_lat, lon + polygon_size_lon],
            [lat + polygon_size_lat, lon + polygon_size_lon],
            [lat + polygon_size_lat, lon + -polygon_size_lon],
            [lat + -polygon_size_lat, lon + -polygon_size_lon]
          ];
          return coords;
        },
        filled: true,
        extruded: true,
        getElevation: d => d.get_avg_traffic_vol() * 20,
        getLineColor: d => {
          if (this.state.mouseover === d.get_id()) {
            return [255, 0, 0];
          }
          return d.get_color();
        },
        wireframe: true,
        pickable: true,
        onClick: e => this.handleOnClickObject(e),
        onHover: e => this.handleOnHover(e),
      })
    ];

    return (
      <div>
        <DeckGL
          {...this.state.viewport}
          {...this.props.mapSize}
          layers={layers}
          controller={MapController}
          onViewportChange={v => this.setState({viewport: v})}
          onLayerClick={e => this.handleOnClickMap(e)}
        >
          <StaticMap
            {...this.props.mapSize}
            mapStyle="mml-basemap.json"
            viewState={this.state.viewport}
          />
        </DeckGL>
       {this.state.tooltipVisible && <Tooltip {...this.state.tooltip} />}
      </div>
    );
  }

}

export default PointMap;
